@@ -19,6 +19,8 @@ class command():
     
     def run(self):
         lastEcho = self.env['runtime']['outputManager'].getLastEcho()
+        if lastEcho.rstrip() != "":
+            lastEcho = lastEcho.rstrip()
         self.env['runtime']['memoryManager'].addValueToFirstIndex('clipboardHistory', lastEcho)
         self.env['runtime']['outputManager'].presentText(lastEcho, soundIcon='CopyToClipboard', interrupt=True)
 
