@@ -26,8 +26,8 @@ class command():
         if currLine.isspace():
             self.env['runtime']['outputManager'].presentText(_("blank"), soundIcon='EmptyLine', interrupt=True)
         else:
-            self.env['runtime']['outputManager'].presentText(currLine[cursorPos['x']], interrupt=True) 
-        self.env['runtime']['outputManager'].announceActiveCursor()            
+            self.env['runtime']['outputManager'].presentText(currLine[cursorPos['x']:], interrupt=True) 
+        self.env['runtime']['outputManager'].announceActiveCursor()
     def setCallback(self, callback):
         pass
 
