@@ -22,6 +22,7 @@ from fenrirscreenreader.core import punctuationManager
 from fenrirscreenreader.core import cursorManager
 from fenrirscreenreader.core import applicationManager
 from fenrirscreenreader.core import helpManager
+from fenrirscreenreader.core import vmenuManager
 from fenrirscreenreader.core import textManager
 from fenrirscreenreader.core import tableManager
 from fenrirscreenreader.core import byteManager
@@ -29,6 +30,7 @@ from fenrirscreenreader.core import attributeManager
 from fenrirscreenreader.core import barrierManager
 from fenrirscreenreader.core import remoteManager
 from fenrirscreenreader.core import sayAllManager
+from fenrirscreenreader.core import quickMenuManager
 from fenrirscreenreader.core import environment 
 from fenrirscreenreader.core.settingsData import settingsData
 from fenrirscreenreader.core import debug
@@ -38,11 +40,14 @@ class settingsManager():
     def __init__(self):
         self.settings = settingsData
         self.settingArgDict = {}
+        self.bindingsBackup = None
+        self.settingsFile = ''
     def initialize(self, environment):
         self.env = environment
     def shutdown(self):
         pass
-        
+    def getBindingBackup(self):
+        return self.bindingsBackup.copy()
     def loadSoundIcons(self, soundIconPath):
         siConfig = open(soundIconPath + '/soundicons.conf',"r")
         while(True):
@@ -71,14 +76,25 @@ class settingsManager():
             self.env['soundIcons'][soundIcon] = soundIconFile
             self.env['runtime']['debug'].writeDebugOut("SoundIcon: " + soundIcon + '.' + soundIconFile, debug.debugLevel.INFO, onAnyLevel=True)               
         siConfig.close()
-    
+    def getSettingsFile(self):
+        return self.settingsFile
+    def setSettingsFile(self, settingsFile):
+        if not os.path.exists(settingsFile):
+            return
+        if not os.access(settingsFile, os.R_OK):
+            return        
+        self.settingsFile = settingsFile
     def loadSettings(self, settingConfigPath):
         if not os.path.exists(settingConfigPath):
             return False
         if not os.access(settingConfigPath, os.R_OK):
             return False
         self.env['settings'] = ConfigParser()
-        self.env['settings'].read(settingConfigPath)
+        try:
+            self.env['settings'].read(settingConfigPath)
+        except:
+            return False
+        self.setSettingsFile(settingConfigPath)
         return True
     def saveSettings(self, settingConfigPath):
         # set opt dict here
@@ -195,6 +211,7 @@ class settingsManager():
                 self.env['input']['scriptKey'].append(key)
     def resetSettingArgDict(self):
         self.settingArgDict = {}
+        self.env['runtime']['outputManager'].resetSpeechDriver()
     def setOptionArgDict(self, section, setting, value):
         #section = section.lower()
         #setting = setting.lower()
@@ -223,18 +240,32 @@ class settingsManager():
             #self.env['runtime']['debug'].writeDebugOut('settingsManager:setOptionArgDict:Datatype missmatch: '+ section + '#' + setting + '=' +  value + ' Error:' +  str(e), debug.debugLevel.ERROR)
             return
 
-
-
     def parseSettingArgs(self, settingArgs):
         for optionElem in settingArgs.split(';'):
-            if len(optionElem.split('#',1)) != 2:
+            settingValList = []
+            sectionOptionList = []
+            section = ''
+            option = ''
+            value = ''
+            settingValList = optionElem.split('=',1)
+            if len(settingValList) != 2:
                 continue
-            if len(optionElem.split('#',1)[1].split('=',1)) != 2:
+            if '#' in settingValList[0]:
+                sectionOptionList = settingValList[0].split('#', 1)
+            elif '.' in settingValList[0]:
+                sectionOptionList = settingValList[0].split('.', 1)
+            elif ',' in settingValList[0]:
+                sectionOptionList = settingValList[0].split(',', 1)
+            elif '!' in settingValList[0]:
+                sectionOptionList = settingValList[0].split('!', 1)
+            else:
+                continue
+            if len(sectionOptionList) != 2:
                 continue
 
-            section = str(optionElem.split('#',1)[0])
-            option = str(optionElem.split('#',1)[1].split('=',1)[0])
-            value = optionElem.split('#',1)[1].split('=',1)[1]
+            section = str(sectionOptionList[0])
+            option = str(sectionOptionList[1])
+            value = str(settingValList[1])
             self.setOptionArgDict(section, option, value)
 
     def initFenrirConfig(self, cliArgs, fenrirManager = None, environment = environment.environment):
@@ -277,7 +308,6 @@ class settingsManager():
             # TODO needs cleanup use dict
             #self.setOptionArgDict('keyboard', 'keyboardLayout', 'pty')
             self.setSetting('keyboard', 'keyboardLayout', 'pty')
-            self.setSetting('general', 'debugFile', '/tmp/fenrir-pty.log')
         if cliArgs.emulated_evdev:
             self.setSetting('screen', 'driver', 'ptyDriver')
             self.setSetting('keyboard', 'driver', 'evdevDriver')
@@ -383,10 +413,21 @@ class settingsManager():
         environment['runtime']['barrierManager'].initialize(environment)
         environment['runtime']['sayAllManager'] = sayAllManager.sayAllManager()
         environment['runtime']['sayAllManager'].initialize(environment)
+        environment['runtime']['vmenuManager'] = vmenuManager.vmenuManager()
+        environment['runtime']['vmenuManager'].initialize(environment)
+        environment['runtime']['quickMenuManager'] = quickMenuManager.quickMenuManager()
+        environment['runtime']['quickMenuManager'].initialize(environment)
+
+        # only possible after having input and screen managers with clean buffer
+        environment['runtime']['inputManager'].writeEventBuffer()
+        environment['runtime']['inputManager'].handleDeviceGrab(force = True)
+
         environment['runtime']['debug'].writeDebugOut('\/-------environment-------\/',debug.debugLevel.INFO, onAnyLevel=True)
         environment['runtime']['debug'].writeDebugOut(str(environment), debug.debugLevel.INFO, onAnyLevel=True)
         environment['runtime']['debug'].writeDebugOut('\/-------settings.conf-------\/', debug.debugLevel.INFO, onAnyLevel=True)
         environment['runtime']['debug'].writeDebugOut(str(environment['settings']._sections) , debug.debugLevel.INFO, onAnyLevel=True)
         environment['runtime']['debug'].writeDebugOut('\/-------self.settingArgDict-------\/',debug.debugLevel.INFO, onAnyLevel=True)
         environment['runtime']['debug'].writeDebugOut(str( self.settingArgDict) ,debug.debugLevel.INFO, onAnyLevel=True)
+        self.bindingsBackup = environment['bindings'].copy()
+
         return environment
