@@ -1,6 +1,3 @@
-#!/bin/python
-# -*- coding: utf-8 -*-
-
 # Fenrir TTY screen reader
 # By Chrys, Storm Dragon, and contributers.
 
@@ -10,7 +7,7 @@ _evdevAvailableError = ''
 _udevAvailableError = ''
 try:
     import evdev
-    from evdev import InputDevice, UInput
+    from evdev import InputDevice, UInput, ecodes as e
     _evdevAvailable = True
 
 except Exception as e:
@@ -43,54 +40,67 @@ class driver(inputDriver):
         self.gDevices = {}
         self.iDeviceNo = 0
         self.watchDog = Value(c_bool, True)
+        self.UInputinject = UInput()
     def initialize(self, environment):
         self.env = environment
         self.env['runtime']['inputManager'].setShortcutType('KEY')
         global _evdevAvailable
         global _udevAvailable
-        self._initialized = _evdevAvailable and _udevAvailable
-        if not self._initialized:
-            global _evdevAvailableError
-            global _udevAvailableError
-            currError = ' '
-            if not _evdevAvailable:
-                currError += _evdevAvailableError
-            if not _udevAvailable:
-                currError += ' ' + _udevAvailableError
-            self.env['runtime']['debug'].writeDebugOut('InputDriver:' + currError, debug.debugLevel.ERROR)
-            return  
+        global _evdevAvailableError
+        global _udevAvailableError
+        if not _udevAvailable:
+            self.env['runtime']['debug'].writeDebugOut('InputDriver:' + _udevAvailableError, debug.debugLevel.ERROR)            
+        if not _evdevAvailable:
+            self.env['runtime']['debug'].writeDebugOut('InputDriver:' + _evdevAvailableError, debug.debugLevel.ERROR)
+            return
 
-        self.env['runtime']['processManager'].addCustomEventThread(self.plugInputDeviceWatchdogUdev)
+        if _udevAvailable:
+            self.env['runtime']['processManager'].addCustomEventThread(self.plugInputDeviceWatchdogUdev)
         self.env['runtime']['processManager'].addCustomEventThread(self.inputWatchdog)
+        self._initialized = True
+
     def plugInputDeviceWatchdogUdev(self,active , eventQueue):
         context = pyudev.Context()
         monitor = pyudev.Monitor.from_netlink(context)
         monitor.filter_by(subsystem='input')
         monitor.start()
+        ignorePlug = False
         while active.value:
             validDevices = []
             device = monitor.poll(1)
             while device:
                 self.env['runtime']['debug'].writeDebugOut('plugInputDeviceWatchdogUdev:' + str(device), debug.debugLevel.INFO)
                 try:
-                    if not '/sys/devices/virtual/input/' in device.sys_path:
+                    try:
+                        if device.name.upper() in ['','SPEAKUP','FENRIR-UINPUT']:
+                            ignorePlug = True
+                        if device.phys.upper() in ['','SPEAKUP','FENRIR-UINPUT']:
+                            ignorePlug = True
+                        if 'BRLTTY' in  device.name.upper():
+                            ignorePlug = True
+                    except Exception as e:
+                        self.env['runtime']['debug'].writeDebugOut("plugInputDeviceWatchdogUdev CHECK NAME CRASH: " + str(e),debug.debugLevel.ERROR)
+                    if not ignorePlug:
+                        virtual = '/sys/devices/virtual/input/' in device.sys_path
                         if device.device_node:
-                            validDevices.append(str(device.device_node))
-                except:                    
-                    pass
+                            validDevices.append({'device': device.device_node, 'virtual': virtual})
+                except Exception as e:
+                    self.env['runtime']['debug'].writeDebugOut("plugInputDeviceWatchdogUdev APPEND CRASH: " + str(e),debug.debugLevel.ERROR)
                 try:
-                    device = monitor.poll(0.1)                
-                except:                    
+                    pollTimeout = 1
+                    device = monitor.poll(pollTimeout)
+                except:
                     device = None
+                ignorePlug = False
             if validDevices:
                 eventQueue.put({"Type":fenrirEventType.PlugInputDevice,"Data":validDevices})
-        return time.time()       
-         
+        return time.time()
+
     def inputWatchdog(self,active , eventQueue):
         try:
             while active.value:
                 r, w, x = select(self.iDevices, [], [], 0.8)
-                event = None                        
+                event = None
                 foundKeyInSequence = False
                 foreward = False
                 eventFired = False
@@ -137,12 +147,13 @@ class driver(inputDriver):
                     if self.gDevices[iDevice.fd]:
                         self.writeUInput(uDevice, event)
             except Exception as e:
-                pass           
+                pass
 
     def writeUInput(self, uDevice, event):
         if not self._initialized:
-            return    
+            return
         uDevice.write_event(event)
+        time.sleep(0.0000002)
         uDevice.syn()
 
     def updateInputDevices(self, newDevices = None, init = False):
@@ -152,6 +163,8 @@ class driver(inputDriver):
         deviceFileList = None
 
         if newDevices and not init:
+            if not isinstance(newDevices, list):
+                newDevices = [newDevices]
             deviceFileList = newDevices
         else:
             deviceFileList = evdev.list_devices()
@@ -188,10 +201,10 @@ class driver(inputDriver):
                 except:
                     continue
                 try:
-                    if currDevice.name.upper() in ['','SPEAKUP','PY-EVDEV-UINPUT']:
-                        continue                
-                    if currDevice.phys.upper() in ['','SPEAKUP','PY-EVDEV-UINPUT']:
-                        continue                    
+                    if currDevice.name.upper() in ['','SPEAKUP','FENRIR-UINPUT']:
+                        continue
+                    if currDevice.phys.upper() in ['','SPEAKUP','FENRIR-UINPUT']:
+                        continue
                     if 'BRLTTY' in  currDevice.name.upper():
                         continue
                 except:
@@ -200,25 +213,25 @@ class driver(inputDriver):
                 if mode in ['ALL','NOMICE']:
                     if eventType.EV_KEY in cap:
                         if 116 in cap[eventType.EV_KEY] and len(cap[eventType.EV_KEY]) < 10:
-                            self.env['runtime']['debug'].writeDebugOut('Device Skipped (has 116):' + currDevice.name,debug.debugLevel.INFO)                                                                
+                            self.env['runtime']['debug'].writeDebugOut('Device Skipped (has 116):' + currDevice.name,debug.debugLevel.INFO)
                             continue
                         if len(cap[eventType.EV_KEY]) < 60:
-                            self.env['runtime']['debug'].writeDebugOut('Device Skipped (< 60 keys):' + currDevice.name,debug.debugLevel.INFO)                                                                                        
-                            continue                                                   
+                            self.env['runtime']['debug'].writeDebugOut('Device Skipped (< 60 keys):' + currDevice.name,debug.debugLevel.INFO)
+                            continue
                         if mode == 'ALL':
                             self.addDevice(currDevice)
-                            self.env['runtime']['debug'].writeDebugOut('Device added (ALL):' + self.iDevices[currDevice.fd].name, debug.debugLevel.INFO)                           
+                            self.env['runtime']['debug'].writeDebugOut('Device added (ALL):' + self.iDevices[currDevice.fd].name, debug.debugLevel.INFO)
                         elif mode == 'NOMICE':
                             if not ((eventType.EV_REL in cap) or (eventType.EV_ABS in cap)):
                                 self.addDevice(currDevice)
-                                self.env['runtime']['debug'].writeDebugOut('Device added (NOMICE):' + self.iDevices[currDevice.fd].name,debug.debugLevel.INFO)                                                
+                                self.env['runtime']['debug'].writeDebugOut('Device added (NOMICE):' + self.iDevices[currDevice.fd].name,debug.debugLevel.INFO)
                             else:
-                                self.env['runtime']['debug'].writeDebugOut('Device Skipped (NOMICE):' + currDevice.name,debug.debugLevel.INFO)      
+                                self.env['runtime']['debug'].writeDebugOut('Device Skipped (NOMICE):' + currDevice.name,debug.debugLevel.INFO)
                     else:
-                        self.env['runtime']['debug'].writeDebugOut('Device Skipped (no EV_KEY):' + currDevice.name,debug.debugLevel.INFO)                                                                                       
+                        self.env['runtime']['debug'].writeDebugOut('Device Skipped (no EV_KEY):' + currDevice.name,debug.debugLevel.INFO)
                 elif currDevice.name.upper() in mode.split(','):
                     self.addDevice(currDevice)
-                    self.env['runtime']['debug'].writeDebugOut('Device added (Name):' + self.iDevices[currDevice.fd].name,debug.debugLevel.INFO) 
+                    self.env['runtime']['debug'].writeDebugOut('Device added (Name):' + self.iDevices[currDevice.fd].name,debug.debugLevel.INFO)
             except Exception as e:
                 self.env['runtime']['debug'].writeDebugOut("Device Skipped (Exception): " + deviceFile +' ' + currDevice.name +' '+ str(e),debug.debugLevel.INFO)
         self.iDeviceNo = len(evdev.list_devices())
@@ -231,12 +244,12 @@ class driver(inputDriver):
                     self.iDevicesFD.append(fd)
             for fd in self.iDevicesFD:
                 if not fd in self.iDevices:
-                    self.iDevicesFD.remove(fd)  
+                    self.iDevicesFD.remove(fd)
         except:
             pass
     def mapEvent(self, event):
         if not self._initialized:
-            return None    
+            return None
         if not event:
             return None
         mEvent = inputData.inputEvent
@@ -251,47 +264,51 @@ class driver(inputDriver):
                             mEvent['EventName'] = mEvent['EventName'][0]
             mEvent['EventValue'] = event.code
             mEvent['EventSec'] = event.sec
-            mEvent['EventUsec'] = event.usec                
+            mEvent['EventUsec'] = event.usec
             mEvent['EventState'] = event.value
             mEvent['EventType']  = event.type
             return mEvent
         except Exception as e:
             return None
-       
+
     def getLedState(self, led = 0):
         if not self.hasIDevices():
-            return False    
+            return False
         # 0 = Numlock
         # 1 = Capslock
         # 2 = Rollen
         for fd, dev in self.iDevices.items():
             if led in dev.leds():
                 return True
-        return False          
+        return False
     def toggleLedState(self, led = 0):
         if not self.hasIDevices():
-            return False    
+            return False
         ledState = self.getLedState(led)
         for i in self.iDevices:
             if self.gDevices[i]:
                 # 17 LEDs
-                if 17 in self.iDevices[i].capabilities():            
+                if 17 in self.iDevices[i].capabilities():
                     if ledState == 1:
                         self.iDevices[i].set_led(led , 0)
                     else:
                         self.iDevices[i].set_led(led , 1)
     def grabAllDevices(self):
         if not self._initialized:
-            return
+            return True
+        ok = True
         for fd in self.iDevices:
-            self.grabDevice(fd)  
-
+            if not self.gDevices[fd]:
+                ok = ok and self.grabDevice(fd)
+        return ok
     def ungrabAllDevices(self):
         if not self._initialized:
-            return
+            return True
+        ok = True
         for fd in self.iDevices:
-            self.ungrabDevice(fd)
-
+            if self.gDevices[fd]:
+                ok = ok and self.ungrabDevice(fd)
+        return ok
     def createUInputDev(self, fd):
         if not self.env['runtime']['settingsManager'].getSettingAsBool('keyboard', 'grabDevices'):
             self.uDevices[fd] = None
@@ -303,48 +320,75 @@ class driver(inputDriver):
             self.uDevices[fd] = None
         if self.uDevices[fd] != None:
             return
-        try:      
-            self.uDevices[fd] = UInput.from_device(self.iDevices[fd])            
+        try:
+            self.uDevices[fd] = UInput.from_device(self.iDevices[fd], name='fenrir-uinput', phys='fenrir-uinput')
         except Exception as e:
             try:
-                self.env['runtime']['debug'].writeDebugOut('InputDriver evdev: compat fallback:  ' + str(e),debug.debugLevel.WARNING)         
+                self.env['runtime']['debug'].writeDebugOut('InputDriver evdev: compat fallback:  ' + str(e),debug.debugLevel.WARNING)
                 dev = self.iDevices[fd]
                 cap = dev.capabilities()
                 del cap[0]
                 self.uDevices[fd] = UInput(
                   cap,
-                  dev.name,
+                  name = 'fenrir-uinput',
+                  phys = 'fenrir-uinput'
                 )
             except Exception as e:
-                self.env['runtime']['debug'].writeDebugOut('InputDriver evdev: init Uinput not possible:  ' + str(e),debug.debugLevel.ERROR)         
-                return               
+                self.env['runtime']['debug'].writeDebugOut('InputDriver evdev: init Uinput not possible:  ' + str(e),debug.debugLevel.ERROR)
+                return
     def addDevice(self, newDevice):
-        self.env['runtime']['debug'].writeDebugOut('InputDriver evdev: device added:  ' + str(newDevice.fd) + ' ' +str(newDevice),debug.debugLevel.INFO)      
-        self.iDevices[newDevice.fd] = newDevice  
-        self.gDevices[newDevice.fd] = False                      
-        self.createUInputDev(newDevice.fd)
+        self.env['runtime']['debug'].writeDebugOut('InputDriver evdev: device added:  ' + str(newDevice.fd) + ' ' +str(newDevice),debug.debugLevel.INFO)
+        try:
+            self.iDevices[newDevice.fd] = newDevice  
+            self.createUInputDev(newDevice.fd)
+            self.gDevices[newDevice.fd] = False
+        except:
+            # if it doesnt work clean up
+            try:
+                del(self.iDevices[newDevice.fd])
+            except:
+                pass
+            try:
+                del(self.uDevices[newDevice.fd])
+            except:
+                pass
+            try:
+                del(self.gDevices[newDevice.fd])
+            except:
+                pass
     def grabDevice(self, fd):
         if not self.env['runtime']['settingsManager'].getSettingAsBool('keyboard', 'grabDevices'):
-            return
+            return True
         try:
             self.iDevices[fd].grab()
             self.gDevices[fd] = True
-            self.env['runtime']['debug'].writeDebugOut('InputDriver evdev: grab device ('+ str(self.iDevices[fd].name) + ')',debug.debugLevel.INFO)            
-        except IOError:            
-            self.gDevices[fd] = True            
+            self.env['runtime']['debug'].writeDebugOut('InputDriver evdev: grab device ('+ str(self.iDevices[fd].name) + ')',debug.debugLevel.INFO)
+        except IOError:
+            if not self.gDevices[fd]:
+                return False
+        #    self.gDevices[fd] = True
+        #    #self.removeDevice(fd)
         except Exception as e:
-            self.env['runtime']['debug'].writeDebugOut('InputDriver evdev: grabing not possible:  ' + str(e),debug.debugLevel.ERROR)         
+            self.env['runtime']['debug'].writeDebugOut('InputDriver evdev: grabing not possible:  ' + str(e),debug.debugLevel.ERROR)
+            return False
+        return True
     def ungrabDevice(self,fd):
         if not self.env['runtime']['settingsManager'].getSettingAsBool('keyboard', 'grabDevices'):
-            return      
+            return True
         try:
-            self.gDevices[fd] = False                    
             self.iDevices[fd].ungrab()
+            self.gDevices[fd] = False
             self.env['runtime']['debug'].writeDebugOut('InputDriver evdev: ungrab device ('+ str(self.iDevices[fd].name) + ')',debug.debugLevel.INFO)
-        except:
-            pass    
+        except IOError:
+            if self.gDevices[fd]:
+                return False
+        #    self.gDevices[fd] = False
+        #    #self.removeDevice(fd)
+        except Exception as e:
+            return False
+        return True
     def removeDevice(self,fd):
-        self.env['runtime']['debug'].writeDebugOut('InputDriver evdev: device removed:  ' + str(fd) + ' ' +str(self.iDevices[fd]),debug.debugLevel.INFO)         
+        self.env['runtime']['debug'].writeDebugOut('InputDriver evdev: device removed:  ' + str(fd) + ' ' +str(self.iDevices[fd]),debug.debugLevel.INFO)
         self.clearEventBuffer()
         try:
             self.ungrabDevice(fd)
@@ -365,13 +409,13 @@ class driver(inputDriver):
         try:
             del(self.uDevices[fd])
         except:
-            pass  
+            pass
         try:
             del(self.gDevices[fd])
         except:
-            pass              
+            pass
         self.updateMPiDevicesFD()
-                 
+
     def hasIDevices(self):
         if not self._initialized:
             return False
@@ -379,7 +423,16 @@ class driver(inputDriver):
             return False
         if len(self.iDevices) == 0:
             return False
-        return True    
+        return True
+
+    def sendKey(self, key, state):
+        if not self._initialized:
+            return
+        try:
+            self.UInputinject.write(e.EV_KEY, e.ecodes[key], state)
+            self.UInputinject.syn()
+        except:
+            pass
 
     def removeAllDevices(self):
         if not self.hasIDevices():
@@ -389,5 +442,5 @@ class driver(inputDriver):
             self.removeDevice(fd)
         self.iDevices.clear()
         self.uDevices.clear()
-        self.gDevices.clear()        
+        self.gDevices.clear()
         self.iDeviceNo = 0 
